﻿using RimWorld.Planet;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using Verse;

namespace SelectableSculptureGraphic {
    class Dialog_SelectTexture : Window {
        protected List<TextureMenuOption> options;

        protected Vector2 scrollPosition;

        protected readonly float ColumnSpacing = 2f;
        protected readonly float RowSpacing = 2f;

        private Vector2 drawSize;

        private int columnCount;

        public override Vector2 InitialSize {
            get {
                return new Vector2(860f, 600f);
            }
        }

        public override bool IsDebug {
            get {
                return true;
            }
        }

        public Vector2 DrawOffset {
            get {
                return drawSize + new Vector2(ColumnSpacing, RowSpacing);
            }
        }

        public Dialog_SelectTexture(IEnumerable<TextureMenuOption> options, Vector2 drawSize) {
            this.optionalTitle = "SelectableSculptureGraphic.SelectTextureTitle".Translate();
            this.doCloseX = true;
            this.onlyOneOfTypeAllowed = true;
            this.absorbInputAroundWindow = true;
            this.draggable = true;
            this.drawSize = drawSize;

            this.options = options.ToList<TextureMenuOption>();
            ResolveColumnCount();
        }

        public override void DoWindowContents(Rect inRect) {
            Rect outRect = new Rect(inRect);
            Rect rect = new Rect(0f, 0f, DrawOffset.x * columnCount, DrawOffset.y * (this.options.Count / columnCount + 1));
            Widgets.BeginScrollView(outRect, ref this.scrollPosition, rect, true);
            this.DoListingItems();
            Widgets.EndScrollView();
        }

        public override void PostClose() {
            base.PostClose();
            UI.UnfocusCurrentControl();
        }

        protected void DoListingItems() {
            for (int i = 0; i < this.options.Count; i++) {
                AddAction(i, this.options[i].texture, this.options[i].method);
            }
        }

        private void AddAction(int index, Texture2D texture, Action<int> action) {
            Rect rect = new Rect(CalcDrawPosition(index),drawSize);
            if (Widgets.ButtonImage(rect,texture)) {
                this.Close(true);
                action(index);
            }
            GUI.color = Color.white;
        }

        //一行あたりの個数計算
        private void ResolveColumnCount() {
            columnCount = 1;
            for (int i=0;i < this.options.Count; i++) {
                if ((columnCount + 1) * DrawOffset.x > InitialSize.x - 40f) {
                    break;
                }
                columnCount++;
            }
        }

        private Vector2 CalcDrawPosition(int index) {
            int count = this.options.Count;
            int x = index % columnCount;
            int y = index / columnCount;
            return new Vector2(x * DrawOffset.x, y * DrawOffset.y);
        }
    }
}
